const express = require("express");
const router = express.Router();
const DecryptWithServerKey = require("./../../services/decryptionservice");
require("dotenv").config();
const StellarSDK = require("stellar-sdk");
const AccountCreateCollection = require("../../model/stellar/accountCreate");
const Practitioner = require("../../model/practitioner/practitioner");
const EncryptionService = require("./../../services/encryptionservice");
const { generateKeyPairSync } = require("crypto");
let timestamp;

router.post("/", async (req, res) => {
  //   const decryptedUserName = DecryptWithServerKey(req.body.username);
  //   const password = req.body.password;
  //   const decryptedEmail = DecryptWithServerKey(req.body.email);
  //   const decryptedWorkspace = DecryptWithServerKey(req.body.workspace);
  //   const decryptedFullName = DecryptWithServerKey(req.body.fullName);

  //get the server key pair
  const serverSourceKeyPair = StellarSDK.Keypair.fromSecret(
    process.env.STELLARSEED
  );
  const serverPubKey = serverSourceKeyPair.publicKey();
  const server = new StellarSDK.Server(process.env.STELLARTESTNET);

  //load server account
  const account = await server.loadAccount(serverPubKey);
  const fee = await server.fetchBaseFee();

  //generate random keypair
  const userKeyPair = StellarSDK.Keypair.random();

  //build the account funding transaction
  const accCreateTransaction = new StellarSDK.TransactionBuilder(account, {
    fee,
    networkPassphrase: StellarSDK.Networks.TESTNET,
  })
    .addMemo(StellarSDK.Memo.text("Create account"))
    .addOperation(
      StellarSDK.Operation.createAccount({
        destination: userKeyPair.publicKey(),
        startingBalance: "10",
      })
    )
    .setTimeout(30)
    .build();

  //sign the transaction
  accCreateTransaction.sign(serverSourceKeyPair);

  try {
    const accCreateTxnResult = await server.submitTransaction(
      accCreateTransaction
    );
    console.log("New user account created: HASH : ", accCreateTxnResult.hash);
    console.log(
      "View account creation transaction at : https://horizon-testnet.stellar.org/transactions/" +
        accCreateTxnResult.hash
    );
    timestamp = new Date();

    //generate RSA key pair
    const keyOptions = [
      {
        modulusLength: 2048,
        publicKeyEncoding: {
          type: "spki",
          format: "pem",
        },
        privateKeyEncoding: {
          type: "pkcs8",
          format: "pem",
          cipher: "aes-256-cbc",
          passphrase: "top secret",
        },
      },
    ];

    const [{ publicKey: RSAPK, privateKey: RSASK }] = keyOptions.map(
      (options) => generateKeyPairSync("rsa", options)
    );

    //base64 encoding of the key
    const b64RSApk = btoa(RSAPK);
    const b64RSAsk = btoa(RSASK);

    //create the account creation details collection object
    const createAccountObj = new AccountCreateCollection({
      publickey: userKeyPair.publicKey(),
      amount: "10XLM",
      timestamp: timestamp,
      transactionhash: accCreateTxnResult.hash,
    });
    //save the account creation details to the database
    try {
      const saveInCollection = await createAccountObj.save();
      console.log(
        "Account creation details saved in the database " + saveInCollection
      );
      //create the practitioner object
      const practitionerObj = new Practitioner({
        username: req.body.username,
        password: req.body.password,
        email: req.body.email,
        fullname: req.body.fullName,
        workspace: req.body.workspace,
        rsakey: b64RSApk,
        publickey: userKeyPair.publicKey(),
        timestamp: timestamp,
      });
      try {
        //save the user collection object to the database
        const saveInPractitionerCollection = await practitionerObj.save();
        console.log(
          "User details added to the DB " + saveInPractitionerCollection
        );
        console.log("Public key: " + userKeyPair.publicKey());
        console.log("Secret: " + userKeyPair.secret());
        res.status(200).json({
          message: "Practitioner details added to the blockchain",
          hash: accCreateTxnResult.hash,
          pubkey: EncryptionService.EncryptWithServerKey(
            userKeyPair.publicKey()
          ),
          seed: EncryptionService.EncryptWithServerKey(userKeyPair.secret()),
          rsapk: EncryptionService.EncryptWithServerKey(b64RSApk),
          rsask: EncryptionService.EncryptWithServerKey(b64RSAsk),
        });
        console.log(
          "/practitioner/register - register practitioner details successful"
        );
      } catch (error) {
        console.log("Error when adding user details to DB : " + error);
        res.status(500).json({
          message: "Error when adding user details to DB: " + error,
        });
        console.log(
          "/practitioner/register - register practitioner details unsuccessful"
        );
      }
    } catch (error) {
      console.log(
        "Error when sending account creation transaction details to DB : " +
          error
      );
      res.status(500).json({
        message:
          "Error when sending account creation transaction details to DB : " +
          error,
      });
      console.log(
        "/practitioner/register - register practitioner details unsuccessful"
      );
    }
  } catch (error) {
    console.log(
      "Error when sending account creation transaction to blockchain: " + error
    );
    res.status(500).json({
      message:
        "Error when sending account creation transaction to blockchain: " +
        error,
    });
    console.log(
      "/practitioner/register - register practitioner details unsuccessful"
    );
  }
});

module.exports = router;
