const StellarSdk = require("stellar-sdk");
require("dotenv").config();
const server = new StellarSdk.Server(process.env.STELLARTESTNET);

async function SendPayment(
  assetCode,
  sourceSeed,
  destinationPublicKey,
  issuerPk
) {
  try {
    // Load source account details
    const sourceKeypair = StellarSdk.Keypair.fromSecret(sourceSeed);
    const sourceAccount = await server.loadAccount(sourceKeypair.publicKey());

    // Load destination account details
    const destinationAccount = await server.loadAccount(destinationPublicKey);

    // Create the payment operation
    const paymentOperation = StellarSdk.Operation.payment({
      destination: destinationPublicKey,
      asset: new StellarSdk.Asset(assetCode, issuerPk),
      amount: "0.0000001",
    });

    // Build the transaction
    const transaction = new StellarSdk.TransactionBuilder(sourceAccount, {
      fee: StellarSdk.BASE_FEE,
      networkPassphrase: StellarSdk.Networks.TESTNET,
    })
      .addOperation(paymentOperation)
      .setTimeout(30)
      .build();

    // Sign the transaction with the source account keypair
    transaction.sign(sourceKeypair);

    // Submit the transaction to the Stellar network
    const transactionResult = await server.submitTransaction(transaction);

    // Log the transaction hash
    console.log(`Transaction hash for payment : ${transactionResult.hash}`);

    return true;
  } catch (error) {
    console.error(error);
    return false;
  }
}
module.exports = SendPayment;
